@@ -1,9 +1,9 @@
 #
 #	Gnu.pm --- The GNU Readline/History Library wrapper module
 #
-#	$Id: Gnu.pm,v 1.90 2002-03-31 00:32:21-05 hiroo Exp $
+#	$Id: Gnu.pm,v 1.98 2009/02/27 12:46:45 hiroo Exp $
 #
-#	Copyright (c) 2001 Hiroo Hayashi.  All rights reserved.
+#	Copyright (c) 2009 Hiroo Hayashi.  All rights reserved.
 #
 #	This program is free software; you can redistribute it and/or
 #	modify it under the same terms as Perl itself.
@@ -58,15 +58,27 @@ These methods are standard methods defined by B<Term::ReadLine>.
 use strict;
 use Carp;
 
+# This module can't be loaded directly.
+BEGIN {
+    if (not defined $Term::ReadLine::VERSION) {
+        croak <<END;
+It is invalid to load Term::ReadLine::Gnu directly.  Please consult
+the Term::ReadLine documentation for more information.
+END
+    }
+}
+
 {
     use Exporter ();
     use DynaLoader;
     use vars qw($VERSION @ISA @EXPORT_OK);
 
-    $VERSION = '1.12';
+    $VERSION = '1.18';
 
     # Term::ReadLine::Gnu::AU makes a function in
     # `Term::ReadLine::Gnu::XS' as a method.
+    # The namespace of Term::ReadLine::Gnu::AU is searched before ones
+    # of other classes
     @ISA = qw(Term::ReadLine::Gnu::AU Term::ReadLine::Stub
 	      Exporter DynaLoader);
 
@@ -83,6 +95,9 @@ use Carp;
 		    RL_STATE_MACROINPUT RL_STATE_MACRODEF
 		    RL_STATE_OVERWRITE RL_STATE_COMPLETING
 		    RL_STATE_SIGHANDLER RL_STATE_UNDOING
+		    RL_STATE_INPUTPENDING RL_STATE_TTYCSAVED
+		    RL_STATE_CALLBACK RL_STATE_VIMOTION
+		    RL_STATE_MULTIKEY RL_STATE_VICMDONCE
 		    RL_STATE_DONE);
 
     bootstrap Term::ReadLine::Gnu $VERSION; # DynaLoader
@@ -93,6 +108,13 @@ require Term::ReadLine::Gnu::XS;
 
 use vars qw(%Attribs %Features);
 
+# Each variable in the GNU Readline Library is tied to an entry of
+# this hash (%Attribs).  By accessing the hash entry, you can read
+# and/or write the variable in the GNU Readline Library.  See the
+# package definition of Term::ReadLine::Gnu::Var and following code
+# for more details.
+
+# Normal (non-tied) entries
 %Attribs  = (
 	     MinLength => 1,
 	     do_expand => 0,
@@ -112,6 +134,9 @@ use vars qw(%Attribs %Features);
 sub Attribs { \%Attribs; }
 sub Features { \%Features; }
 
+# keep rl_readline_version value for efficiency
+my $readline_version;
+
 #
 #	GNU Readline/History Library constant definition
 #	These are included in @EXPORT_OK.
@@ -160,7 +185,15 @@ sub RL_STATE_OVERWRITE		{ 0x02000; } # overwrite mode
 sub RL_STATE_COMPLETING		{ 0x04000; } # doing completion
 sub RL_STATE_SIGHANDLER		{ 0x08000; } # in readline sighandler
 sub RL_STATE_UNDOING		{ 0x10000; } # doing an undo
-sub RL_STATE_DONE		{ 0x80000; } # done; accepted line
+# The following RL_STATE_* are defined since TRL 6.0
+sub RL_STATE_INPUTPENDING	{ 0x020000; } #	rl_execute_next called
+sub RL_STATE_TTYCSAVED		{ 0x040000; } #	tty special chars saved
+sub RL_STATE_CALLBACK		{ 0x080000; } #	using the callback interface
+sub RL_STATE_VIMOTION		{ 0x100000; } #	reading vi motion arg
+sub RL_STATE_MULTIKEY		{ 0x200000; } #	reading multiple-key command
+sub RL_STATE_VICMDONCE		{ 0x400000; } #	entered vi command mode at least once
+# The value was changed since TRL 6.0
+sub RL_STATE_DONE		{ $readline_version < 0x0600 ? 0x80000 : 0x800000; } # done; accepted line
 
 #
 #	Methods Definition
@@ -220,12 +253,17 @@ sub new {
 	my ($IN,$OUT) = $self->findConsole();
 	open(IN,"<$IN")   || croak "Cannot open $IN for read";
 	open(OUT,">$OUT") || croak "Cannot open $OUT for write";
+	# borrowed from Term/ReadLine.pm
+	my $sel = select(OUT);
+	$| = 1;				# for DB::OUT
+	select($sel);
 	$Attribs{instream} = \*IN;
 	$Attribs{outstream} = \*OUT;
     } else {
 	$Attribs{instream} = shift;
 	$Attribs{outstream} = shift;
     }
+    $readline_version = $Attribs{readline_version};
 
     $self;
 }
@@ -295,7 +333,7 @@ sub readline {			# should be ReadLine
 	($result, $line) = $self->history_expand($line);
 	my $outstream = $Attribs{outstream};
 	print $outstream "$line\n" if ($result);
-     
+
 	# return without adding line into history
 	if ($result < 0 || $result == 2) {
 	    return '';		# don't return `undef' which means EOF.
@@ -303,7 +341,7 @@ sub readline {			# should be ReadLine
     }
 
     # add to history buffer
-    $self->add_history($line) 
+    $self->add_history($line)
 	if (defined $self->{MinLength} && $self->{MinLength} > 0
 	    && length($line) >= $self->{MinLength});
 
@@ -351,7 +389,7 @@ sub MinLine {
     $self->{MinLength} = shift;
     $old_minlength;
 }
-    
+
 # findConsole is defined in ReadLine.pm.
 
 =item C<findConsole>
@@ -476,7 +514,8 @@ use vars qw(%_rl_vars);
        history_search_delimiter_chars		=> ['S', 12],
        rl_executing_macro			=> ['S', 13], # GRL4.2
        history_word_delimiters			=> ['S', 14], # GRL4.2
-       
+       rl_display_prompt			=> ['S', 15], # GRL6.0
+
        rl_point					=> ['I', 0],
        rl_end					=> ['I', 1],
        rl_mark					=> ['I', 2],
@@ -492,24 +531,33 @@ use vars qw(%_rl_vars);
        history_length				=> ['I', 12],
        history_max_entries			=> ['I', 13],
        max_input_history			=> ['I', 13], # before GRL 4.2
-       history_expansion_char			=> ['C', 14],
-       history_subst_char			=> ['C', 15],
-       history_comment_char			=> ['C', 16],
-       history_quotes_inhibit_expansion		=> ['I', 17],
-       rl_erase_empty_line			=> ['I', 18], # GRL 4.0
-       rl_catch_signals				=> ['I', 19], # GRL 4.0
-       rl_catch_sigwinch			=> ['I', 20], # GRL 4.0
-       rl_already_prompted			=> ['I', 21], # GRL 4.1
-       rl_num_chars_to_read			=> ['I', 22], # GRL 4.2
-       rl_dispatching				=> ['I', 23], # GRL 4.2
-       rl_gnu_readline_p			=> ['I', 24], # GRL 4.2
-       rl_readline_state			=> ['I', 25], # GRL 4.2
-       rl_explicit_arg				=> ['I', 26], # GRL 4.2
-       rl_numeric_arg				=> ['I', 27], # GRL 4.2
-       rl_editing_mode				=> ['I', 28], # GRL 4.2
-       rl_attempted_completion_over		=> ['I', 29], # GRL 4.2
-       rl_completion_type			=> ['I', 30], # GRL 4.2
-       rl_readline_version			=> ['I', 31], # GRL 4.2a
+       history_write_timestamps			=> ['I', 14], # GRL 5.0
+       history_expansion_char			=> ['C', 15],
+       history_subst_char			=> ['C', 16],
+       history_comment_char			=> ['C', 17],
+       history_quotes_inhibit_expansion		=> ['I', 18],
+       rl_erase_empty_line			=> ['I', 19], # GRL 4.0
+       rl_catch_signals				=> ['I', 20], # GRL 4.0
+       rl_catch_sigwinch			=> ['I', 21], # GRL 4.0
+       rl_already_prompted			=> ['I', 22], # GRL 4.1
+       rl_num_chars_to_read			=> ['I', 23], # GRL 4.2
+       rl_dispatching				=> ['I', 24], # GRL 4.2
+       rl_gnu_readline_p			=> ['I', 25], # GRL 4.2
+       rl_readline_state			=> ['I', 26], # GRL 4.2
+       rl_explicit_arg				=> ['I', 27], # GRL 4.2
+       rl_numeric_arg				=> ['I', 28], # GRL 4.2
+       rl_editing_mode				=> ['I', 29], # GRL 4.2
+       rl_attempted_completion_over		=> ['I', 30], # GRL 4.2
+       rl_completion_type			=> ['I', 31], # GRL 4.2
+       rl_readline_version			=> ['I', 32], # GRL 4.2a
+       rl_completion_suppress_append		=> ['I', 33], # GRL 4.3
+       rl_completion_quote_character		=> ['C', 34], # GRL 5.0
+       rl_completion_suppress_quote		=> ['I', 35], # GRL 5.0
+       rl_completion_found_quote		=> ['I', 36], # GRL 5.0
+       rl_completion_mark_symlink_dirs		=> ['I', 37], # GRL 4.3
+       rl_prefer_env_winsize			=> ['I', 38], # GRL 5.1
+       rl_sort_completion_matches		=> ['I', 39], # GRL 6.0
+       rl_completion_invoking_key		=> ['C', 40], # GRL 6.0
 
        rl_startup_hook				=> ['F', 0],
        rl_event_hook				=> ['F', 1],
@@ -525,8 +573,9 @@ use vars qw(%_rl_vars);
        history_inhibit_expansion_function	=> ['F', 11],
        rl_pre_input_hook			=> ['F', 12], # GRL 4.0
        rl_completion_display_matches_hook	=> ['F', 13], # GRL 4.0
-       rl_prep_term_function			=> ['F', 14], # GRL 4.2
-       rl_deprep_term_function			=> ['F', 15], # GRL 4.2
+       rl_completion_word_break_hook		=> ['F', 14], # GRL 5.0
+       rl_prep_term_function			=> ['F', 15], # GRL 4.2
+       rl_deprep_term_function			=> ['F', 16], # GRL 4.2
 
        rl_instream				=> ['IO', 0],
        rl_outstream				=> ['IO', 1],
@@ -552,7 +601,7 @@ sub FETCH {
 	confess "Term::ReadLine::Gnu::Var::FETCH: Unknown variable name `$name'\n";
 	return undef ;
     }
-    
+
     my ($type, $id) = @{$_rl_vars{$name}};
     if ($type eq 'S') {
 	return _rl_fetch_str($id);
@@ -583,7 +632,7 @@ sub STORE {
 	confess "Term::ReadLine::Gnu::Var::STORE: Unknown variable name `$name'\n";
 	return undef ;
     }
-    
+
     my $value = shift;
     my ($type, $id) = @{$_rl_vars{$name}};
     if ($type eq 'S') {
@@ -627,7 +676,7 @@ foreach (keys %Term::ReadLine::Gnu::Var::_rl_vars) {
 #	add reference to some functions
 {
     my ($name, $fname);
-    no strict 'refs';		# allow symbolic refernce
+    no strict 'refs';		# allow symbolic reference
     map {
 	($name = $_) =~ s/^rl_//; # strip leading `rl_'
 	$fname = 'Term::ReadLine::Gnu::XS::' . $_;
@@ -753,6 +802,11 @@ Bind C<KEY> to the C<FUNCTION>.  C<FUNCTION> is the name added by the
 C<add_defun> method.  If optional argument C<MAP> is specified, binds
 in C<MAP>.  Returns non-zero in case of error.
 
+=item C<bind_key_if_unbound(KEY, FUNCTION [,MAP])>
+
+	int	rl_bind_key_if_unbound(int key, FunctionPtr|str function,
+			    	       Keymap|str map = rl_get_keymap()) #GRL5.0
+
 =item C<unbind_key(KEY [,MAP])>
 
 	int	rl_unbind_key(int key, Keymap|str map = rl_get_keymap())
@@ -769,10 +823,20 @@ Bind C<KEY> to the null function.  Returns non-zero in case of error.
 	int	rl_unbind_command(str command,
 				  Keymap|str map = rl_get_keymap())
 
+=item C<bind_keyseq(KEYSEQ, FUNCTION [,MAP])>
+
+	int	rl_bind_keyseq(str keyseq, FunctionPtr|str function,
+			       Keymap|str map = rl_get_keymap()) # GRL 5.0
+
 =item C<set_key(KEYSEQ, FUNCTION [,MAP])>
 
 	int	rl_set_key(str keyseq, FunctionPtr|str function,
-				  Keymap|str map = rl_get_keymap())
+			   Keymap|str map = rl_get_keymap())
+
+=item C<bind_keyseq_if_unbound(KEYSEQ, FUNCTION [,MAP])>
+
+	int	rl_bind_keyseq_if_unbound(str keyseq, FunctionPtr|str function,
+					  Keymap|str map = rl_get_keymap()) # GRL 5.0
 
 =item C<generic_bind(TYPE, KEYSEQ, DATA, [,MAP])>
 
@@ -995,6 +1059,10 @@ detail see 'GNU Readline Library Manual'.
 
 	void	rl_tty_set_default_bindings([Keymap KMAP])	# GRL 4.2
 
+=item C<tty_unset_default_bindings(KMAP)>
+
+	void	rl_tty_unset_default_bindings([Keymap KMAP])	# GRL 5.0
+
 =item C<reset_terminal([TERMINAL_NAME])>
 
 	int	rl_reset_terminal(str terminal_name = getenv($TERM)) # GRL 4.2
@@ -1005,6 +1073,20 @@ detail see 'GNU Readline Library Manual'.
 
 =over 4
 
+=item C<save_state(READLINE_STATE)>
+
+	NOT IMPLEMENTED YET!
+	int	rl_save_state(struct readline_state *sp)	# GRL 6.0
+
+=item C<restore_state(READLINE_STATE)>
+
+	NOT IMPLEMENTED YET!
+	int	rl_restore_state(struct readline_state *sp)	# GRL 6.0
+
+=item C<replace_line(TEXT [,CLEAR_UNDO])>
+
+	int	rl_replace_line(str text, int clear_undo)	# GRL 4.3
+
 =item C<initialize>
 
 	int	rl_initialize()
@@ -1045,6 +1127,10 @@ When C<MAX> is ommited, the max length of an item in @matches is used.
 
 	int	rl_variable_bind(const str variable, const str value)
 
+=item C<variable_value(VARIABLE)>
+
+	str	rl_variable_value(const str variable)	# GRL 5.1
+
 =item C<variable_dumper(READABLE)>
 
 	int	rl_variable_dumper(int readline)
@@ -1095,6 +1181,10 @@ When C<MAX> is ommited, the max length of an item in @matches is used.
 
 	void	rl_reset_after_signal()	# GRL 4.0
 
+=item C<echo_signal_char>
+
+	void	rl_echo_signal_char(int sig)	# GRL 6.0
+
 =item C<resize_terminal>
 
 	void	rl_resize_terminal()	# GRL 4.0
@@ -1107,6 +1197,10 @@ When C<MAX> is ommited, the max length of an item in @matches is used.
 
 	(int rows, int cols)	rl_get_screen_size()	# GRL 4.2
 
+=item C<reset_screen_size()>
+
+	void	rl_reset_screen_size()	# GRL 5.1
+
 =item C<set_signals>
 
 	int	rl_set_signals()	# GRL 4.0
@@ -1125,6 +1219,10 @@ When C<MAX> is ommited, the max length of an item in @matches is used.
 
 	int	rl_complete_internal(int what_to_do = TAB)
 
+=item C<completion_mode(FUNCTION)>
+
+	int	rl_completion_mode(FunctionPtr|str function)
+
 =item C<completion_matches(TEXT [,FUNC])>
 
 	(@str)	rl_completion_matches(str text,
@@ -1185,6 +1283,10 @@ This is equivalent with 'stifle_history(undef)'.
 sets the history of input, from where it can be used if the actual
 C<readline> is present.
 
+=item C<add_history_time(STRING)>
+
+	void	add_history_time(str string)	# GRL 5.0
+
 =item C<remove_history(WHICH)>
 
 	str	remove_history(int which)
@@ -1219,6 +1321,10 @@ C<readline> is present.
 
 	str	history_get(offset)
 
+=item C<history_get_time(OFFSET)>
+
+	time_t	history_get_time(offset)
+
 =item C<history_total_bytes>
 
 	int	history_total_bytes()
@@ -1359,12 +1465,13 @@ Examples:
 	int rl_point
 	int rl_end
 	int rl_mark
-	int rl_done		
+	int rl_done
 	int rl_num_chars_to_read (GRL 4.2)
 	int rl_pending_input
 	int rl_dispatching (GRL 4.2)
 	int rl_erase_empty_line (GRL 4.0)
 	str rl_prompt (read only)
+	str rl_display_prompt (GRL 6.0)
 	int rl_already_prompted (GRL 4.1)
 	str rl_library_version (read only)
 	int rl_readline_version (read only)
@@ -1373,6 +1480,8 @@ Examples:
 	str rl_readline_name
 	filehandle rl_instream
 	filehandle rl_outstream
+	int rl_prefer_env_winsize (GRL 5.1)
+	pfunc rl_last_func (GRL 4.2)
 	pfunc rl_startup_hook
 	pfunc rl_pre_input_hook (GRL 4.0)
 	pfunc rl_event_hook
@@ -1380,7 +1489,6 @@ Examples:
 	pfunc rl_redisplay_function
 	pfunc rl_prep_term_function (GRL 4.2)
 	pfunc rl_deprep_term_function (GRL 4.2)
-	pfunc rl_last_func (GRL 4.2)
 	Keymap rl_executing_keymap (read only)
 	Keymap rl_binding_keymap (read only)
 	str rl_executing_macro (GRL 4.2)
@@ -1405,15 +1513,23 @@ Examples:
 	str rl_basic_word_break_characters
 	str rl_basic_quote_characters
 	str rl_completer_word_break_characters
+	pfunc rl_completion_word_break_hook (GRL 5.0)
 	str rl_completer_quote_characters
 	str rl_filename_quote_characters
 	str rl_special_prefixes
 	int rl_completion_append_character
+	int rl_completion_suppress_append (GRL 4.3)
+	int rl_completion_quote_charactor (GRL 5.0)
+	int rl_completion_suppress_quote (GRL 5.0)
+	int rl_completion_found_quote (GRL 5.0)
+	int rl_completion_mark_symlink_dirs (GRL 4.3)
 	int rl_ignore_completion_duplicates
 	int rl_filename_completion_desired
 	int rl_filename_quoting_desired
 	int rl_attempted_completion_over (GRL 4.2)
+	int rl_sort_completion_matches (GRL 6.0)
 	int rl_completion_type (GRL 4.2)
+	int rl_completion_invoking_key (GRL 6.0)
 	int rl_inhibit_completion
 	pfunc rl_ignore_some_completion_function
 	pfunc rl_directory_completion_hook
@@ -1424,6 +1540,7 @@ Examples:
 	int history_base
 	int history_length
 	int history_max_entries (called `max_input_history'. read only)
+	int history_write_timestamps (GRL 5.0)
 	char history_expansion_char
 	char history_subst_char
 	char history_comment_char
@@ -1803,6 +1920,12 @@ It provides a friendly way to play with the Ghostscript interpreter,
 including command history and auto-completion of Postscript font names
 and reserved words.
 
+=item vshnu (the New Visual Shell)
+
+	http://www.cs.indiana.edu/~kinzler/vshnu/
+
+A visual shell and CLI shell supplement.
+
 =back
 
 If you know any other works which can be listed here, please let me
